import os

from nbconvert.exporters import MarkdownExporter
from nbconvert.filters.strings import path2url
from traitlets import default
from traitlets.config import Config


class JekyllExporter(MarkdownExporter):
    """Exporter to write Markdown with Jekyll metadata"""

    resources = {}

    def from_filename(self, filename, resources=None, **kwargs):
        """Convert notebook from a file

        Args:
            filename (str): Full filename of the notebook to convert.
            resources (dict): Additional resources used by preprocessors and filters.
            `**kwargs`: Ignored

        Returns:
            NotebookNode: Converted notebook.
            dict: Resources dictionary.
        """
        self.resources = resources
        return super(JekyllExporter, self).from_filename(
            filename, resources=resources, **kwargs
        )

    @default("template_file")
    def _template_file_default(self):  # skipcq: PYL-R0201
        """Default template file"""
        return "jekyll"

    @property
    def template_path(self):
        """Path to available template files"""
        return super(JekyllExporter, self).template_path + [
            os.path.join(os.path.dirname(__file__), "templates")
        ]

    @property
    def preprocessors(self):
        """Add JekyllPreprocessor to list of enabled preprocessors"""
        return super(JekyllExporter, self).preprocessors + [
            "jekyllnb.JekyllPreprocessor"
        ]

    @property
    def default_config(self):
        """Default configuration"""
        config = Config({"JekyllPreprocessor": {"enabled": True}})
        config.merge(super(JekyllExporter, self).default_config)
        return config

    def default_filters(self):
        """Default filters"""
        for pair in super(JekyllExporter, self).default_filters():
            yield pair

        image_dir = self.resources.get("image_dir", self.resources["output_files_dir"])

        # convert image path to one compatible with Jekyll
        yield (
            "jekyllpath",
            lambda path: "{{ site.baseurl }}/"
            + path2url(os.path.join(image_dir, os.path.basename(path))),
        )



from nbconvert.preprocessors import Preprocessor


class JekyllPreprocessor(Preprocessor):
    """Preprocessor to add Jekyll metadata"""

    def preprocess(self, nb, resources):  # skipcq: PYL-R0201
        """Preprocess notebook

        Add Jekyll metadata to notebook resources.

        Args:
            nb (NotebookNode): Notebook being converted.
            resources (dict): Additional resources used by preprocessors and filters.

        Returns:
            NotebookNode: Modified notebook.
            dict: Modified resources dictionary.
        """
        name = resources["metadata"]["name"]
        metadata = {"layout": "page", "title": name, "permalink": "/" + name}
        metadata.update(nb.metadata.get("jekyll", {}))
        resources["metadata"]["jekyll"] = metadata

        return nb, resources



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jekyllnb']

package_data = \
{'': ['*'], 'jekyllnb': ['templates/*']}

install_requires = \
['nbconvert>=5.4,<6.0']

entry_points = \
{'console_scripts': ['jupyter-jekyllnb = jekyllnb:JekyllNB.launch_instance'],
 'nbconvert.exporters': ['jekyll = jekyllnb:JekyllExporter']}

setup_kwargs = {
    'name': 'jekyllnb',
    'version': '0.1.2',
    'description': 'Convert Jupyter Notebooks to Jekyll-ready Markdown files',
    'long_description': '# JekyllNB: Jupyter Notebooks to Jekyll Markdown\n\n[![Test Status](https://github.com/klane/jekyllnb/workflows/Tests/badge.svg)](https://github.com/klane/jekyllnb/actions)\n[![Documentation Status](https://img.shields.io/readthedocs/jekyllnb.svg?label=Docs&logo=read%20the%20docs)](https://jekyllnb.readthedocs.io/en/latest)\n[![Coverage Status](https://img.shields.io/codecov/c/github/klane/jekyllnb.svg?label=Coverage&logo=codecov)](https://codecov.io/gh/klane/jekyllnb)\n[![PyPI Version](https://img.shields.io/pypi/v/jekyllnb?label=Version&logo=pypi)](https://pypi.org/project/jekyllnb)\n[![Dependabot Status](https://api.dependabot.com/badges/status?host=github&repo=klane/jekyllnb)](https://dependabot.com)\n[![License](https://img.shields.io/github/license/klane/jekyllnb.svg?label=License)](LICENSE)\n[![Code Style](https://img.shields.io/badge/Code%20Style-black-000000.svg)](https://github.com/psf/black)\n\nJekyllNB extends Jupyter\'s command line tool `nbconvert` to add the Jekyll header to Markdown files and save generated images to a desired location.\nThis allows you to easily convert all your notebooks to the required format and immediately build your Jekyll site.\n\n## Installation\n\nJekyllNB is available on PyPI and can be installed with `pip`.\n\n```bash\npip install jekyllnb\n```\n\n## Usage\n\nJekyllNB is a Jupyter app just like `nbconvert`. Call it with `jupyter jekyllnb`.\nThe preprocessor reads metadata from your notebook to populate the Jekyll header.\nAdd a `jekyll` section to your notebook metadata similar to:\n\n```json\n"jekyll": {\n    "layout": "notebook",\n    "permalink": "/hello/",\n    "title": "Hello World!"\n}\n```\n\nThe exporter will add the following header to the generated Markdown:\n\n```text\n---\nlayout: notebook\npermalink: /hello/\ntitle: Hello World!\n---\n```\n\n## Options\n\nSince `jekyllnb` extends `nbconvert`, all existing options are supported. The following new options are available:\n\n- `--site-dir`: Root directory of your Jekyll site. Markdown (`page-dir`) and image (`image-dir`) folders will be created here if they do not exist.\n- `--page-dir`: Directory for generated Markdown files (e.g. _pages or _posts).\n- `--image-dir`: Directory for images. Images are organized into folders for each notebook by default.\nAlias for the `nbconvert` option `NbConvertApp.output_files_dir`.\n- `--no-auto-folder`: Flag to turn off the default behavior of organizing images by notebook name within `image-dir`. (default: `false`)\n\n## nbconvert\n\nJekyllNB also supports `nbconvert` by registering an entry point for the exporter.\nYou can use the Jekyll exporter with `nbconvert` by calling `jupyter nbconvert --to jekyll`.\n\n**Note**: The options above are not available with `nbconvert`.\n',
    'author': 'Kevin Lane',
    'author_email': 'lane.kevin.a@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/klane/jekyllnb',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
}


setup(**setup_kwargs)



import os

from nbconvert.nbconvertapp import NbConvertApp, nbconvert_aliases, nbconvert_flags
from traitlets import Bool, Unicode, default, observe
from traitlets.config import catch_config_error

from .__version__ import __version__

JEKYLLNB_ALIASES = {}
JEKYLLNB_ALIASES.update(nbconvert_aliases)
JEKYLLNB_ALIASES.update(
    {
        "site-dir": "JekyllNB.site_dir",
        "page-dir": "JekyllNB.page_dir",
        "image-dir": "NbConvertApp.output_files_dir",
    }
)

JEKYLLNB_FLAGS = {}
JEKYLLNB_FLAGS.update(nbconvert_flags)
JEKYLLNB_FLAGS.update(
    {
        "no-auto-folder": (
            {"JekyllNB": {"auto_folder": False}},
            "Do not create image folder with notebook name at image-dir level.",
        )
    }
)


class JekyllNB(NbConvertApp):
    """Application to convert notebooks (``*.ipynb``) to Jekyll Markdown (``.md``)"""

    name = "jupyter-jekyllnb"
    description = "Convert Jupyter notebooks to Jekyll-ready Markdown"
    version = __version__

    aliases = JEKYLLNB_ALIASES
    flags = JEKYLLNB_FLAGS

    auto_folder = Bool(True).tag(config=True)
    site_dir = Unicode("").tag(config=True)
    page_dir = Unicode("").tag(config=True)

    @default("export_format")
    def _export_format_default(self):  # skipcq: PYL-R0201
        """Default export format"""
        return "jekyll"

    @observe("export_format")
    def _export_format_changed(self, change):
        """Ensure export format is jekyll"""
        default_format = self._export_format_default()

        if change["new"].lower() != default_format:
            raise ValueError(
                "Invalid export format {}, value must be {}".format(
                    change["new"], default_format
                )
            )

    @catch_config_error
    def initialize(self, argv=None):
        """Initialize application, notebooks, writer, and postprocessor"""
        super(JekyllNB, self).initialize(argv)
        self.writer.build_directory = os.path.join(self.site_dir, self.page_dir)

        if self.auto_folder:
            self.output_files_dir = os.path.join(
                self.output_files_dir, "{notebook_name}"
            )

    def init_single_notebook_resources(self, notebook_filename):
        """Initialize resources

        Initializes the resources dictionary for a single notebook.

        Args:
            notebook_filename (str): Full filename of the notebook to convert.

        Returns:
            dict: Resources dictionary for a single notebook.

            Dictionary must include the following keys:
                - config_dir: location of the Jupyter config directory
                - unique_key: notebook name
                - output_files_dir: directory where output files should be saved
        """
        resources = super(JekyllNB, self).init_single_notebook_resources(
            notebook_filename
        )
        resources["image_dir"] = resources["output_files_dir"]
        resources["output_files_dir"] = os.path.join(
            os.getcwd(), self.site_dir, resources["output_files_dir"]
        )

        return resources
